/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.pooling;

import java.util.Collection;
import java.util.Iterator;

/**
 * {@link PoolData}
 *
 * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
 * @author <a href="mailto:daniel.becker@open-xchange.com">Daniel Becker</a> - extraction
 * @param <T> The class of the {@link PooledData}
 * @since v7.10.1
 */
public interface PoolData<T> {

    /**
     * @return the topmost idle object or <code>null</code> if the idle stack is
     *         empty.
     */
    PooledData<T> popIdle();

    /**
     * Adds a new active object.
     * 
     * @param newActive new active object to add.
     */
    void addActive(final PooledData<T> newActive);

    PooledData<T> getActive(final T pooled);

    Iterator<PooledData<T>> listActive();

    /**
     * Removes an active object.
     * 
     * @param toRemove object to remove.
     * @return <code>true</code> if the object was removed successfully.
     */
    boolean removeActive(final PooledData<T> toRemove);

    void addCreating();

    void removeCreating();

    int getCreating();

    /**
     * @return the number of active objects.
     */
    int numActive();

    boolean isActiveEmpty();

    void addIdle(final PooledData<T> newIdle);

    PooledData<T> getIdle(final int index);

    void removeIdle(final int index);

    /**
     * @return the number of idle objects.
     */
    int numIdle();

    boolean isIdleEmpty();

    void addByThread(final PooledData<T> toCheck);

    PooledData<T> getByThread(final Thread thread);

    void removeByThread(final PooledData<T> toCheck);

    Collection<PooledData<T>> getActive();
}
