/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.mail.autoconfig.json.actions;

import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.exception.OXException;
import com.openexchange.java.Strings;
import com.openexchange.java.util.Tools;
import com.openexchange.mail.autoconfig.Autoconfig;
import com.openexchange.mail.autoconfig.AutoconfigService;
import com.openexchange.mail.oauth.MailOAuthService;
import com.openexchange.server.ServiceExceptionCode;
import com.openexchange.server.ServiceLookup;
import com.openexchange.tools.servlet.AjaxExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link GetAction}
 *
 * @author <a href="mailto:martin.herfurth@open-xchange.com">Martin Herfurth</a>
 */
public class GetAction extends AutoconfigAction {

    private static final String EMAIL = "email";

    private static final String PASSWORD = "password";

    private static final String FORCE_SECURE = "force_secure";

    private static final String OAUTH = "oauth";

    /**
     * Initializes a new {@link GetAction}.
     *
     * @param services
     */
    public GetAction(ServiceLookup services) {
        super(services);
    }

    @Override
    public AJAXRequestResult perform(AJAXRequestData request, ServerSession session) throws OXException {
        if (request.containsParameter(OAUTH)) {
            String sOAuth = request.getParameter(OAUTH);
            int id = parseInt(sOAuth);
            if (id >= 0) {
                MailOAuthService mailOAuthService = getService(MailOAuthService.class);
                if (null == mailOAuthService) {
                    throw ServiceExceptionCode.absentService(MailOAuthService.class);
                }

                Autoconfig autoconfig = mailOAuthService.getAutoconfigFor(id, session);
                return new AJAXRequestResult(autoconfig, "autoconfig");
            }
        }

        String mail = request.getParameter(EMAIL, String.class);
        String password = request.getParameter(PASSWORD);

        boolean forceSecure = true;
        if (request.containsParameter(FORCE_SECURE)) {
            forceSecure = request.getParameter(FORCE_SECURE, Boolean.class).booleanValue();
        }

        if (Strings.isEmpty(password)) {
            throw AjaxExceptionCodes.MISSING_PARAMETER.create(PASSWORD);
        }

        AutoconfigService autoconfigService = getAutoconfigService();
        Autoconfig autoconfig = autoconfigService.getConfig(mail, password, session.getUserId(), session.getContextId(), forceSecure);
        return new AJAXRequestResult(autoconfig, "autoconfig");
    }

    private int parseInt(String str) {
        return Tools.getUnsignedInteger(str);
    }

}
