/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.mail.autoconfig.json.converter;

import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.ajax.requesthandler.Converter;
import com.openexchange.ajax.requesthandler.ResultConverter;
import com.openexchange.exception.OXException;
import com.openexchange.mail.autoconfig.Autoconfig;
import com.openexchange.tools.servlet.AjaxExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link AutoconfigResultConverter}
 *
 * @author <a href="mailto:martin.herfurth@open-xchange.com">Martin Herfurth</a>
 */
public class AutoconfigResultConverter implements ResultConverter {

    @Override
    public String getInputFormat() {
        return "autoconfig";
    }

    @Override
    public String getOutputFormat() {
        return "json";
    }

    @Override
    public Quality getQuality() {
        return Quality.GOOD;
    }

    @Override
    public void convert(AJAXRequestData requestData, AJAXRequestResult result, ServerSession session, Converter converter) throws OXException {
        result.setResultObject(convert((Autoconfig) result.getResultObject()), "json");
    }

    private JSONObject convert(Autoconfig autoconfig) throws OXException {
        if (autoconfig == null) {
            return new JSONObject(1);
        }

        try {
            boolean noTransport = null == autoconfig.getTransportServer();
            JSONObject json = new JSONObject(16);
            json.put("login", autoconfig.getUsername());
            json.put("transport_login", noTransport ? null : autoconfig.getUsername());
            json.put("mail_server", autoconfig.getMailServer());
            json.put("transport_server", autoconfig.getTransportServer());
            json.put("mail_port", autoconfig.getMailPort());
            json.put("transport_port", autoconfig.getTransportPort());
            json.put("mail_protocol", autoconfig.getMailProtocol());
            json.put("transport_protocol", autoconfig.getTransportProtocol());
            json.put("mail_secure", autoconfig.isMailSecure());
            json.put("transport_secure", autoconfig.isTransportSecure());
            json.put("mail_starttls", autoconfig.isMailStartTls());
            json.put("transport_starttls", noTransport ? null :  Boolean.valueOf(autoconfig.isTransportStartTls()));
            Integer mailOAuthId = autoconfig.getMailOAuthId();
            if (null != mailOAuthId) {
                json.put("mail_oauth", mailOAuthId);
            }
            Integer transportOAuthId = autoconfig.getTransportOAuthId();
            if (null != transportOAuthId) {
                json.put("transport_oauth", transportOAuthId);
            }
            String sourceName = autoconfig.getSource();
            if (null != sourceName) {
                json.put("config_source", sourceName);
            }
            return json;
        } catch (JSONException e) {
            throw AjaxExceptionCodes.JSON_ERROR.create(e.getMessage());
        }
    }

}
