/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.publish.database;

import java.sql.Connection;
import java.util.concurrent.atomic.AtomicReference;
import com.openexchange.database.provider.DBTransactionPolicy;
import com.openexchange.database.provider.SimpleDBProvider;
import com.openexchange.datatypes.genericonf.storage.GenericConfigurationStorageService;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.delete.DeleteEvent;
import com.openexchange.groupware.delete.DeleteListener;
import com.openexchange.publish.PublicationStorage;
import com.openexchange.publish.PublicationTargetDiscoveryService;
import com.openexchange.publish.sql.PublicationSQLStorage;


/**
 * Upon the deletion of a user, the user's publications are also deleted.
 *
 * @author <a href="mailto:tobias.prinz@open-xchange.com">Tobias Prinz</a>
 */
public class PublicationUserDeleteListener implements DeleteListener {

    private final AtomicReference<PublicationTargetDiscoveryService> discoveryService;
    private final AtomicReference<GenericConfigurationStorageService> genConfStorage;

    /**
     * Initializes a new {@link PublicationUserDeleteListener}.
     */
    public PublicationUserDeleteListener() {
        super();
        discoveryService = new AtomicReference<PublicationTargetDiscoveryService>(null);
        genConfStorage = new AtomicReference<GenericConfigurationStorageService>(null);
    }

    @Override
    public void deletePerformed(final DeleteEvent event, final Connection readCon, final Connection writeCon) throws OXException {
        Context context = event.getContext();
        PublicationStorage store = getStorage(writeCon);
        int type = event.getType();
        if (type == DeleteEvent.TYPE_USER) {
            store.deletePublicationsOfUser(event.getId(), context);
        } else if (type == DeleteEvent.TYPE_CONTEXT) {
            store.deletePublicationsInContext(context.getContextId(), context);
        }
    }

    protected PublicationStorage getStorage(Connection writeCon) {
        return new PublicationSQLStorage(new SimpleDBProvider(writeCon, writeCon), DBTransactionPolicy.NO_TRANSACTIONS, genConfStorage.get(), discoveryService.get());
    }

    public void setDiscoveryService(PublicationTargetDiscoveryService discoveryService) {
        this.discoveryService.set(discoveryService);
    }

    public void setGenConfStorage(GenericConfigurationStorageService genConfStorage) {
        this.genConfStorage.set(genConfStorage);
    }

}
