/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.groupware.contexts.impl;

import com.openexchange.groupware.contexts.Context;

/**
 * ContextExtended - an extended version of <code>Context</code> interface providing additional methods
 * <ul>
 * <li>{@link #setUpdating(boolean)}</li>
 * <li>{@link #setReadOnly(boolean)}</li>
 * </ul>
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public interface ContextExtended extends Context {

    /**
     * Sets context's update status. This causes sessions that belong to a
     * updating context to die as fast as possible to be able to maintain these
     * contexts.
     * @param updating the context's update status
     */
    void setUpdating(boolean updating);

    /**
     * Marks the context as read only. This can be used to prevent writing operations before getting a database down exception.
     * @param readOnly the context's read only status.
     */
    void setReadOnly(boolean readOnly);

    /**
     * Checks if context-associated database schema needs an update since there are pending update tasks.
     *
     * @return <code>true</code> if update is needed; otherwise <code>false</code>
     */
    boolean isUpdateNeeded();

    /**
     * Sets that context-associated database schema needs an update since there are pending update tasks. However, actual update has not yet
     * been triggered.
     *
     * @param updateNeeded The update-needed flag to set
     */
    void setUpdateNeeded(boolean updateNeeded);

}
